from django.core.management.base import BaseCommand

from rest_framework_jwt.blacklist.models import BlacklistedToken


class Command(BaseCommand):
    help = 'Deletes any expired Blacklisted tokens'

    def handle(self, *args, **kwargs):
        BlacklistedToken.objects.delete_stale_tokens()



from rest_framework.permissions import IsAuthenticated
from rest_framework.viewsets import ModelViewSet

from rest_framework_jwt.blacklist.models import BlacklistedToken

from ..authentication import JSONWebTokenAuthentication
from .serializers import BlacklistTokenSerializer


class BlacklistView(ModelViewSet):
    queryset = BlacklistedToken.objects.all()
    serializer_class = BlacklistTokenSerializer
    permission_classes = (IsAuthenticated, )

    def create(self, request, *args, **kwargs):
        if 'token' not in request.data:
            request.data.update({
                'token': JSONWebTokenAuthentication.get_token_from_request(request)
            })

        return super(BlacklistView, self).create(request, *args, **kwargs)



# -*- coding: utf-8 -*-

from django.contrib import admin

from .models import BlacklistedToken


@admin.register(BlacklistedToken)
class BlacklistedTokenAdmin(admin.ModelAdmin):
    list_display = ('user', 'token', 'expires_at', 'blacklisted_at', )
    list_filter = ('user', )



# -*- coding: utf-8 -*-

from django.conf import settings
from django.db import models
from django.utils import timezone


class BlacklistedTokenManager(models.Manager):
    def delete_stale_tokens(self):
        return self.filter(expires_at__lt=timezone.now()).delete()


class BlacklistedToken(models.Model):
    token = models.TextField(db_index=True, unique=True)
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
    expires_at = models.DateTimeField(db_index=True)
    blacklisted_at = models.DateTimeField(auto_now_add=True)

    objects = BlacklistedTokenManager()

    def __str__(self):
        return 'Blacklisted token - {} - {}'.format(self.user, self.token)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from datetime import timedelta

from django.core.management import call_command

from django.utils import timezone

from rest_framework import status
from rest_framework.reverse import reverse

from rest_framework_jwt.blacklist.models import BlacklistedToken
from rest_framework_jwt.settings import api_settings


def test_anonymous_user_cannot_blacklist_tokens(api_client):
    url = reverse('blacklist-list')
    response = api_client.post(url)

    assert response.status_code == status.HTTP_401_UNAUTHORIZED

    response = api_client.post(url, {'user': 1})

    assert response.status_code == status.HTTP_401_UNAUTHORIZED


def test_user_can_blacklist_own_token(
    user, create_authenticated_client
):
    api_client = create_authenticated_client(user)

    url = reverse('blacklist-list')
    response = api_client.post(url)

    assert response.status_code == status.HTTP_201_CREATED
    assert BlacklistedToken.objects.exists()

    test_url = reverse('blacklist-test-view')

    response = api_client.get(test_url)
    assert response.status_code == status.HTTP_403_FORBIDDEN


def test_user_cannot_blacklist_same_token_multiple_times(
    user, create_authenticated_client
):
    url = reverse('blacklist-list')

    api_client = create_authenticated_client(user)
    api_client.post(url)

    assert BlacklistedToken.objects.count() == 1

    response = api_client.post(url)

    assert response.status_code == status.HTTP_403_FORBIDDEN
    assert BlacklistedToken.objects.count() == 1


def test_user_can_blacklist_own_token_from_cookie(
    monkeypatch, user, call_auth_endpoint
):
    auth_cookie = "jwt-auth"
    monkeypatch.setattr(api_settings, "JWT_AUTH_COOKIE", auth_cookie)

    api_client = call_auth_endpoint("username", "password")

    url = reverse('blacklist-list')
    response = api_client.client.post(url)

    assert response.status_code == status.HTTP_201_CREATED
    assert BlacklistedToken.objects.exists()

    test_url = reverse('blacklist-test-view')

    response = api_client.client.get(test_url)
    assert response.status_code == status.HTTP_403_FORBIDDEN


def test_stale_tokens_are_deleted_on_token_save_when_feature_is_activated(
    monkeypatch, user, staff_user, create_authenticated_client
):
    blacklisted_token = BlacklistedToken.objects.create(
        token='stale_token',
        user=user,
        expires_at=timezone.now(),
    )
    assert BlacklistedToken.objects.exists()
    assert blacklisted_token.token == 'stale_token'

    blacklisted_token.expires_at = blacklisted_token.expires_at - timedelta(days=2)
    blacklisted_token.save()

    url = reverse('blacklist-list')
    api_client = create_authenticated_client(user)

    response = api_client.post(url)

    assert response.status_code == status.HTTP_201_CREATED
    # No token is deleted because the feature is not turned on
    assert BlacklistedToken.objects.count() == 2

    monkeypatch.setattr(api_settings, 'JWT_DELETE_STALE_BLACKLISTED_TOKENS', True)

    api_client = create_authenticated_client(staff_user)

    response = api_client.post(url)

    assert response.status_code == status.HTTP_201_CREATED
    assert BlacklistedToken.objects.count() == 2


def test_delete_stale_tokens_by_calling_the_management_command(
    user, create_authenticated_client
):
    for num in range(1, 6):
        BlacklistedToken.objects.create(
            token='stale_token_{}'.format(num),
            user=user,
            expires_at=timezone.now() - timedelta(days=num),
        )
    assert BlacklistedToken.objects.count() == 5

    BlacklistedToken.objects.create(
        token='valid_token',
        user=user,
        expires_at=timezone.now() + timedelta(days=1),
    )
    assert BlacklistedToken.objects.count() == 6

    call_command('delete_stale_tokens')
    assert BlacklistedToken.objects.count() == 1



# -*- coding: utf-8 -*-

from django.apps import AppConfig


class BlacklistedTokenConfig(AppConfig):
    name = 'rest_framework_jwt.blacklist'

    def ready(self):
        import rest_framework_jwt.blacklist.signals  # noqa



# -*- coding: utf-8 -*-

from rest_framework.exceptions import AuthenticationFailed

from rest_framework_jwt.compat import gettext_lazy as _


class MissingToken(AuthenticationFailed):
    status_code = 401
    msg = _('The token is missing.')
    default_code = 'token_missing'


class InvalidAuthorizationCredentials(AuthenticationFailed):
    status_code = 401
    msg = _('Invalid Authorization header.')
    default_code = 'invalid_authorization_credentials'


class InvalidAuthorizationHeaderPrefix(AuthenticationFailed):
    status_code = 401
    msg = _('Authentication credentials were not provided.')
    default_code = 'invalid_authorization_header_prefix'



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from base64 import b64encode

from collections import OrderedDict

from django.utils.encoding import force_str
from django.utils.translation import ugettext_lazy as _

from jwt import get_unverified_header
from jwt.exceptions import InvalidKeyError, InvalidAlgorithmError, InvalidSignatureError

from pytest import skip, fixture, raises

from rest_framework import status

from rest_framework_jwt.authentication import JSONWebTokenAuthentication
from rest_framework_jwt.compat import gettext_lazy as _
from rest_framework_jwt.compat import has_set_cookie_samesite
from rest_framework_jwt.settings import api_settings

from sys import version_info

@fixture
def rsa_keys(scope="session"):
    from cryptography.hazmat.backends import default_backend
    from cryptography.hazmat.primitives.asymmetric import rsa

    nkeys = 2
    secret = {}
    public = {}

    for i in range(1, nkeys + 1):
        name = "rsa%d" % i
        secret[name] = rsa.generate_private_key(
            public_exponent=65537,
            # key_size=512 is probably unsafe for any real world code!
            key_size=512,
            backend=default_backend()
        )
        public[name] = secret[name].public_key()

    rsa_keys = {"secret": secret, "public": public}
    return rsa_keys

def test_empty_credentials_returns_validation_error(call_auth_endpoint):
    expected_output = {
        "password": [_("This field may not be blank.")],
        "username": [_("This field may not be blank.")],
    }

    response = call_auth_endpoint("", "")

    assert response.json() == expected_output


def test_auth__invalid_credentials__returns_validation_error(
    call_auth_endpoint,
):
    expected_output = {
        "non_field_errors": [_("Unable to log in with provided credentials.")]
    }

    response = call_auth_endpoint("invalid_username", "invalid_password")

    assert response.json() == expected_output


def test_valid_credentials_return_jwt(user, call_auth_endpoint):
    response = call_auth_endpoint("username", "password")

    token = response.json()["token"]
    payload = JSONWebTokenAuthentication.jwt_decode_token(token)

    assert response.status_code == status.HTTP_200_OK
    assert payload["user_id"] == user.id
    assert payload["username"] == user.get_username()


def test_valid_credentials_with_aud_and_iss_settings_return_jwt(
    monkeypatch, user, call_auth_endpoint
):
    monkeypatch.setattr(api_settings, "JWT_AUDIENCE", "test-aud")
    monkeypatch.setattr(api_settings, "JWT_ISSUER", "test-iss")

    response = call_auth_endpoint("username", "password")

    token = response.json()["token"]
    payload = JSONWebTokenAuthentication.jwt_decode_token(token)

    assert response.status_code == status.HTTP_200_OK
    assert payload["aud"] == "test-aud"
    assert payload["iss"] == "test-iss"
    assert payload["user_id"] == user.id
    assert payload["username"] == user.get_username()


def test_valid_credentials_with_JWT_GET_USER_SECRET_KEY_set_return_jwt(
    monkeypatch, user, call_auth_endpoint
):
    def jwt_get_user_secret_key(user):
        return "{0}-{1}-{2}".format(user.pk, user.get_username(), "key")

    monkeypatch.setattr(
        api_settings, "JWT_GET_USER_SECRET_KEY", jwt_get_user_secret_key
    )

    response = call_auth_endpoint("username", "password")

    token = response.json()["token"]
    payload = JSONWebTokenAuthentication.jwt_decode_token(token)

    assert response.status_code == status.HTTP_200_OK
    assert payload["user_id"] == user.id
    assert payload["username"] == user.get_username()


def test_valid_credentials_with_no_user_id_setting_returns_jwt(
    monkeypatch, user, call_auth_endpoint
):
    monkeypatch.setattr(api_settings, "JWT_PAYLOAD_INCLUDE_USER_ID", False)

    response = call_auth_endpoint("username", "password")

    token = response.json()["token"]
    payload = JSONWebTokenAuthentication.jwt_decode_token(token)

    assert response.status_code == status.HTTP_200_OK
    assert "user_id" not in payload
    assert payload["username"] == user.get_username()


def test_valid_credentials_inactive_user_return_validation_error(
    create_user, call_auth_endpoint
):
    expected_output = {
        "non_field_errors": [_("Unable to log in with provided credentials.")]
    }

    create_user(username="inactive", password="password", is_active=False)

    response = call_auth_endpoint("inactive", "password")

    assert response.json() == expected_output


def test_valid_credentials_with_auth_cookie_enabled_returns_jwt_and_cookie(
    monkeypatch, user, call_auth_endpoint
):

    auth_cookie = "jwt-auth"
    monkeypatch.setattr(api_settings, "JWT_AUTH_COOKIE", auth_cookie)

    response = call_auth_endpoint("username", "password")

    assert auth_cookie in response.cookies

    setcookie = response.cookies[auth_cookie]

    assert 'domain' not in setcookie.items()
    assert setcookie['path'] == '/'
    assert setcookie['secure'] is True
    assert setcookie['httponly'] is True	# hardcoded
    if has_set_cookie_samesite():
        assert setcookie['samesite'] == 'Lax'

    assert response.status_code == status.HTTP_200_OK
    assert "token" in force_str(response.content)
    assert auth_cookie in response.client.cookies


def test_auth_cookie_settings(
    monkeypatch, user, call_auth_endpoint
):

    auth_cookie = "jwt-auth"
    monkeypatch.setattr(api_settings, "JWT_AUTH_COOKIE", auth_cookie)
    monkeypatch.setattr(api_settings, "JWT_AUTH_COOKIE_DOMAIN", '.do.main')
    monkeypatch.setattr(api_settings, "JWT_AUTH_COOKIE_PATH", '/pa/th')
    monkeypatch.setattr(api_settings, "JWT_AUTH_COOKIE_SECURE", False)
    monkeypatch.setattr(api_settings, "JWT_AUTH_COOKIE_SAMESITE", 'Strict')

    response = call_auth_endpoint("username", "password")

    assert auth_cookie in response.cookies

    setcookie = response.cookies[auth_cookie]

    assert setcookie['domain'] == '.do.main'
    assert setcookie['path'] == '/pa/th'
    assert 'secure' not in setcookie.items()
    assert setcookie['httponly'] is True	# hardcoded
    if has_set_cookie_samesite():
        assert setcookie['samesite'] == 'Strict'


def test_multi_keys_hash_hash(
    monkeypatch, user, call_auth_endpoint
):
    monkeypatch.setattr(api_settings, "JWT_SECRET_KEY", ["key1", "key2"])

    response = call_auth_endpoint("username", "password")
    token = response.json()["token"]

    monkeypatch.setattr(api_settings, "JWT_SECRET_KEY", ["key2", "key1"])

    payload = JSONWebTokenAuthentication.jwt_decode_token(token)

    assert response.status_code == status.HTTP_200_OK
    assert payload["user_id"] == user.id
    assert payload["username"] == user.get_username()


def test_multi_keys_rsa_rsa(
    monkeypatch, user, call_auth_endpoint, rsa_keys
):

    monkeypatch.setattr(api_settings, "JWT_ALGORITHM", "RS256")
    monkeypatch.setattr(
        api_settings, "JWT_PUBLIC_KEY", list(rsa_keys["public"].values())
    )

    for skey in rsa_keys["secret"].values():
        monkeypatch.setattr(api_settings, "JWT_PRIVATE_KEY", skey)

        response = call_auth_endpoint("username", "password")
        token = response.json()["token"]

        payload = JSONWebTokenAuthentication.jwt_decode_token(token)

        assert response.status_code == status.HTTP_200_OK
        assert payload["user_id"] == user.id
        assert payload["username"] == user.get_username()


def test_signing_and_acceptance_of_multiple_algorithms(
    monkeypatch, user, call_auth_endpoint, rsa_keys
):

    monkeypatch.setattr(
        api_settings, "JWT_PRIVATE_KEY", rsa_keys["secret"]["rsa1"]
    )
    monkeypatch.setattr(
        api_settings, "JWT_PUBLIC_KEY", rsa_keys["public"]["rsa1"]
    )

    for algo in [["HS256", "RS256"], ["RS256", "HS256"]]:
        monkeypatch.setattr(api_settings, "JWT_ALGORITHM", algo)

        response = call_auth_endpoint("username", "password")
        token = response.json()["token"]

        # check needs to succeed no matter which algo is first
        algo = [algo[1], algo[0]]
        monkeypatch.setattr(api_settings, "JWT_ALGORITHM", algo)
        payload = JSONWebTokenAuthentication.jwt_decode_token(token)

        assert response.status_code == status.HTTP_200_OK
        assert payload["user_id"] == user.id
        assert payload["username"] == user.get_username()

        # changing the signature raises exception
        token += "a"
        with raises(InvalidSignatureError):
            assert JSONWebTokenAuthentication.jwt_decode_token(token) == None



def test_keys_with_key_id(
    monkeypatch, user, call_auth_endpoint, rsa_keys
):

    monkeypatch.setattr(
        api_settings, "JWT_PRIVATE_KEY", { "rsa1": rsa_keys["secret"]["rsa1"] }
    )
    monkeypatch.setattr(
        api_settings, "JWT_PUBLIC_KEY", rsa_keys["public"]
    )

    secret_key = OrderedDict([("hash1", "one"), ("hash2", "two")])
    monkeypatch.setattr(api_settings, "JWT_SECRET_KEY", secret_key)

    for kid, algo in {"hash1": ["HS256", "RS256"], "rsa1": ["RS256", "HS256"]}.items():
        monkeypatch.setattr(api_settings, "JWT_ALGORITHM", algo)

        response = call_auth_endpoint("username", "password")
        token = response.json()["token"]

        # check needs to succeed no matter which algo is first
        algo = [algo[1], algo[0]]
        monkeypatch.setattr(api_settings, "JWT_ALGORITHM", algo)
        payload = JSONWebTokenAuthentication.jwt_decode_token(token)
        hdr = get_unverified_header(token)

        assert hdr["kid"] == kid
        assert response.status_code == status.HTTP_200_OK
        assert payload["user_id"] == user.id
        assert payload["username"] == user.get_username()

def test_keys_key_id_not_found(
    monkeypatch, user, call_auth_endpoint
):

    secret_key = OrderedDict([("hash1", "one"), ("hash2", "two")])
    monkeypatch.setattr(api_settings, "JWT_SECRET_KEY", secret_key)

    response = call_auth_endpoint("username", "password")
    token = response.json()["token"]

    secret_key = OrderedDict(hash3="three")
    monkeypatch.setattr(api_settings, "JWT_SECRET_KEY", secret_key)

    with raises(InvalidKeyError):
        assert JSONWebTokenAuthentication.jwt_decode_token(token) == None

def test_insist_on_key_id(
    monkeypatch, user, call_auth_endpoint
):

    secret = "averybadone"

    monkeypatch.setattr(api_settings, "JWT_SECRET_KEY", secret)

    response = call_auth_endpoint("username", "password")
    assert response.status_code == status.HTTP_200_OK

    token = response.json()["token"]

    # check if key is still accepted when giving it a name
    monkeypatch.setattr(api_settings, "JWT_SECRET_KEY", {"kid": secret})

    payload = JSONWebTokenAuthentication.jwt_decode_token(token)
    assert payload["user_id"] == user.id
    assert payload["username"] == user.get_username()

    # check if we insist on the key beging named
    monkeypatch.setattr(api_settings, "JWT_INSIST_ON_KID", True)
    with raises(InvalidKeyError):
        assert JSONWebTokenAuthentication.jwt_decode_token(token) == None

def test_InvalidAlgorithmError():

    hdr = '{"alg": "bad", "typ": "JWT"}'
    jwt = b64encode(hdr.encode("ascii")) + "..".encode("ascii")

    with raises(InvalidAlgorithmError):
        assert JSONWebTokenAuthentication.jwt_decode_token(jwt) == None



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from calendar import timegm
from datetime import datetime

import jwt

from django.apps import apps
from django.contrib.auth import get_user_model
from django.utils.encoding import force_str

from rest_framework import serializers
from rest_framework.utils.encoders import JSONEncoder

from rest_framework_jwt.blacklist.models import BlacklistedToken
from rest_framework_jwt.compat import gettext_lazy as _
from rest_framework_jwt.settings import api_settings


def unix_epoch(datetime_object=None):
    """Get unix epoch from datetime object."""

    if not datetime_object:
        datetime_object = datetime.utcnow()
    return timegm(datetime_object.utctimetuple())


def get_username_field():
    return get_user_model().USERNAME_FIELD


def jwt_get_secret_key(payload=None):
    """
    For enhanced security you may want to use a secret key based on user.

    This way you have an option to logout only this user if:
        - token is compromised
        - password is changed
        - etc.
    """

    if api_settings.JWT_GET_USER_SECRET_KEY:
        username = api_settings.JWT_PAYLOAD_GET_USERNAME_HANDLER(payload)
        User = get_user_model()
        
        # Make sure user exists
        try:
            user = User.objects.get_by_natural_key(username)
        except User.DoesNotExist:
            msg = _("User doesn't exist.")
            raise serializers.ValidationError(msg)
        
        key = api_settings.JWT_GET_USER_SECRET_KEY(user)
        return key
    return api_settings.JWT_SECRET_KEY


def jwt_create_payload(user):
    """
    Create JWT claims token.

    To be more standards-compliant please refer to the official JWT standards
    specification: https://tools.ietf.org/html/rfc7519#section-4.1
    """

    issued_at_time = datetime.utcnow()
    expiration_time = issued_at_time + api_settings.JWT_EXPIRATION_DELTA

    payload = {
        'username': user.get_username(),
        'iat': unix_epoch(issued_at_time),
        'exp': expiration_time
    }

    if api_settings.JWT_PAYLOAD_INCLUDE_USER_ID:
        payload['user_id'] = user.pk

    # It's common practice to have user object attached to profile objects.
    # If you have some other implementation feel free to create your own
    # `jwt_create_payload` method with custom payload.
    if hasattr(user, 'profile'):
        payload['user_profile_id'] = user.profile.pk if user.profile else None,

    # Include original issued at time for a brand new token
    # to allow token refresh
    if api_settings.JWT_ALLOW_REFRESH:
        payload['orig_iat'] = unix_epoch(issued_at_time)

    if api_settings.JWT_AUDIENCE is not None:
        payload['aud'] = api_settings.JWT_AUDIENCE

    if api_settings.JWT_ISSUER is not None:
        payload['iss'] = api_settings.JWT_ISSUER

    return payload


def jwt_get_username_from_payload_handler(payload):
    """
    Override this function if username is formatted differently in payload
    """

    return payload.get('username')


def jwt_encode_payload(payload):
    """Encode JWT token claims."""

    headers=None

    signing_algorithm = api_settings.JWT_ALGORITHM
    if isinstance(signing_algorithm,list):
        signing_algorithm = signing_algorithm[0]
    if signing_algorithm.startswith("HS"):
        key = jwt_get_secret_key(payload)
    else:
        key = api_settings.JWT_PRIVATE_KEY

    if isinstance(key, dict):
        kid, key = next(iter(key.items()))
        headers = {"kid": kid}
    elif isinstance(key,list):
        key = key[0]

    return jwt.encode(payload, key, signing_algorithm, headers=headers, json_encoder=JSONEncoder).decode()


def jwt_decode_token(token):
    """Decode JWT token claims."""

    options = {
        'verify_exp': api_settings.JWT_VERIFY_EXPIRATION,
    }

    algorithms = api_settings.JWT_ALGORITHM
    if not isinstance(algorithms, list):
        algorithms = [algorithms]

    hdr = jwt.get_unverified_header(token)
    alg_hdr = hdr["alg"]
    if alg_hdr not in algorithms:
        raise jwt.exceptions.InvalidAlgorithmError

    kid = hdr["kid"] if "kid" in hdr else None

    keys = None
    if alg_hdr.startswith("HS"):
        unverified_payload = jwt.decode(token, None, False)
        keys = jwt_get_secret_key(unverified_payload)
    else:
        keys = api_settings.JWT_PUBLIC_KEY

    # if keys are named and the jwt has a kid, only consider exactly that key
    # otherwise if the JWT has no kid, JWT_INSIST_ON_KID selects if we fail
    # or try all defined keys
    if isinstance(keys, dict):
        if kid:
            try:
                keys = keys[kid]
            except KeyError:
                raise jwt.exceptions.InvalidKeyError
        elif api_settings.JWT_INSIST_ON_KID:
            raise jwt.exceptions.InvalidKeyError
        else:
            keys = list(keys.values())

    if not isinstance(keys, list):
        keys = [keys]

    ex = None
    for key in keys:
        try:
            return jwt.decode(
                token, key, api_settings.JWT_VERIFY, options=options,
                leeway=api_settings.JWT_LEEWAY,
                audience=api_settings.JWT_AUDIENCE,
                issuer=api_settings.JWT_ISSUER, algorithms=[alg_hdr]
            )
        except (jwt.exceptions.InvalidSignatureError) as e:
                ex = e
    raise ex


def jwt_create_response_payload(
        token, user=None, request=None, issued_at=None
):
    """
    Return data ready to be passed to serializer.

    Override this function if you need to include any additional data for
    serializer.

    Note that we are using `pk` field here - this is for forward compatibility
    with drf add-ons that might require `pk` field in order (eg. jsonapi).
    """
    return {'pk': issued_at, 'token': token}


def check_payload(token):
    from rest_framework_jwt.authentication import JSONWebTokenAuthentication

    if apps.is_installed('rest_framework_jwt.blacklist'):
        if BlacklistedToken.objects.filter(token=force_str(token)).exists():
            msg = _('Token is blacklisted.')
            raise serializers.ValidationError(msg)

    try:
        payload = JSONWebTokenAuthentication.jwt_decode_token(token)
    except jwt.ExpiredSignature:
        msg = _('Token has expired.')
        raise serializers.ValidationError(msg)
    except jwt.DecodeError:
        msg = _('Error decoding token.')
        raise serializers.ValidationError(msg)

    return payload


def check_user(payload):
    from rest_framework_jwt.authentication import JSONWebTokenAuthentication

    username = JSONWebTokenAuthentication. \
        jwt_get_username_from_payload(payload)

    if not username:
        msg = _('Invalid token.')
        raise serializers.ValidationError(msg)

    # Make sure user exists
    try:
        User = get_user_model()
        user = User.objects.get_by_natural_key(username)
    except User.DoesNotExist:
        msg = _("User doesn't exist.")
        raise serializers.ValidationError(msg)

    if not user.is_active:
        msg = _('User account is disabled.')
        raise serializers.ValidationError(msg)

    return user



from rest_framework.permissions import BasePermission

from rest_framework_jwt.authentication import JSONWebTokenAuthentication
from rest_framework_jwt.blacklist.models import BlacklistedToken
from rest_framework_jwt.compat import gettext_lazy as _


class IsNotBlacklisted(BasePermission):
    message = _('You have been blacklisted.')

    def has_permission(self, request, view):
        return not BlacklistedToken.objects.filter(
            token=JSONWebTokenAuthentication.get_token_from_request(request)
        ).exists()



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from datetime import datetime

from django.utils.timezone import make_aware

from rest_framework import serializers
from rest_framework.exceptions import APIException

from rest_framework_jwt.authentication import JSONWebTokenAuthentication
from rest_framework_jwt.blacklist.models import BlacklistedToken
from rest_framework_jwt.settings import api_settings
from rest_framework_jwt.utils import check_user, unix_epoch


class BlacklistTokenSerializer(serializers.ModelSerializer):
    """
    Serializer used for blacklisting tokens.
    """

    class Meta:
        model = BlacklistedToken
        fields = ('token', )
        extra_kwargs = {'token': {'required': False}}

    def save(self, **kwargs):
        token = self.validated_data.get('token')

        payload = JSONWebTokenAuthentication.jwt_decode_token(token)

        iat = payload.get('iat', unix_epoch())
        expires_at_unix_time = iat + api_settings.JWT_EXPIRATION_DELTA.total_seconds()

        self.validated_data.update({
            'user': check_user(payload),
            'expires_at':
                make_aware(datetime.utcfromtimestamp(expires_at_unix_time)),
        })
        return super(BlacklistTokenSerializer, self).save(**kwargs)



# -*- coding: utf-8 -*-

from django.db.models.signals import post_save
from django.dispatch import receiver

from .models import BlacklistedToken
from ..settings import api_settings


@receiver(post_save, sender=BlacklistedToken)
def delete_stale_tokens(sender, **kwargs):
    if api_settings.JWT_DELETE_STALE_BLACKLISTED_TOKENS:
        BlacklistedToken.objects.delete_stale_tokens()



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import jwt

from django.apps import apps
from django.contrib.auth import get_user_model
from django.utils.encoding import force_str

from rest_framework import exceptions
from rest_framework.authentication import (
    BaseAuthentication,
    get_authorization_header,
)

from rest_framework_jwt.blacklist.exceptions import (
    InvalidAuthorizationCredentials,
    InvalidAuthorizationHeaderPrefix,
    MissingToken,
)
from rest_framework_jwt.blacklist.models import BlacklistedToken
from rest_framework_jwt.compat import gettext_lazy as _
from rest_framework_jwt.compat import smart_str
from rest_framework_jwt.settings import api_settings


class JSONWebTokenAuthentication(BaseAuthentication):
    """
    Token based authentication using the JSON Web Token standard.

    Clients should authenticate by passing the token key in the "Authorization"
    HTTP header, prepended with the string specified in the setting
    `JWT_AUTH_HEADER_PREFIX`. For example:

        Authorization: Bearer eyJhbGciOiAiSFMyNTYiLCAidHlwIj
    """

    www_authenticate_realm = 'api'

    @classmethod
    def jwt_create_payload(cls, *args, **kwargs):
        return api_settings.JWT_PAYLOAD_HANDLER(*args, **kwargs)

    @classmethod
    def jwt_encode_payload(cls, *args, **kwargs):
        return api_settings.JWT_ENCODE_HANDLER(*args, **kwargs)

    @classmethod
    def jwt_decode_token(cls, *args, **kwargs):
        return api_settings.JWT_DECODE_HANDLER(*args, **kwargs)

    @classmethod
    def jwt_get_username_from_payload(cls, *args, **kwargs):
        return api_settings.JWT_PAYLOAD_GET_USERNAME_HANDLER(*args, **kwargs)

    @classmethod
    def jwt_create_response_payload(cls, *args, **kwargs):
        return api_settings.JWT_RESPONSE_PAYLOAD_HANDLER(*args, **kwargs)

    def authenticate(self, request):
        """
        Returns a two-tuple of `User` and token if a valid signature has been
        supplied using JWT-based authentication.  Otherwise returns `None`.
        """
        try:
            token = self.get_token_from_request(request)
        except MissingToken:
            return None

        if apps.is_installed('rest_framework_jwt.blacklist'):
            if BlacklistedToken.objects.filter(token=force_str(token)).exists():
                msg = _('Token is blacklisted.')
                raise exceptions.PermissionDenied(msg)

        try:
            payload = self.jwt_decode_token(token)
        except jwt.ExpiredSignature:
            msg = _('Token has expired.')
            raise exceptions.AuthenticationFailed(msg)
        except jwt.DecodeError:
            msg = _('Error decoding token.')
            raise exceptions.AuthenticationFailed(msg)

        user = self.authenticate_credentials(payload)

        return user, token

    @classmethod
    def get_token_from_request(cls, request):
        authorization_header = force_str(get_authorization_header(request))

        try:
            return cls.get_token_from_authorization_header(authorization_header)
        except InvalidAuthorizationHeaderPrefix as error:
            raise exceptions.AuthenticationFailed(error.msg)
        except InvalidAuthorizationCredentials:
            return cls.get_token_from_cookies(request.COOKIES)

    @classmethod
    def get_token_from_authorization_header(cls, authorization_header):
        try:
            prefix, token = authorization_header.split(' ')
        except ValueError:
            raise InvalidAuthorizationCredentials
        else:
            if not cls.prefixes_match(prefix):
                raise InvalidAuthorizationHeaderPrefix
            if not token:
                raise InvalidAuthorizationCredentials
            return token

    @classmethod
    def prefixes_match(cls, prefix):
        authorization_header_prefix = api_settings.JWT_AUTH_HEADER_PREFIX.lower()

        return smart_str(prefix.lower()) == authorization_header_prefix

    @classmethod
    def get_token_from_cookies(cls, cookies):
        if api_settings.JWT_IMPERSONATION_COOKIE:
            imp_user_token = cookies.get(api_settings.JWT_IMPERSONATION_COOKIE)
            if imp_user_token:
                return imp_user_token

        if api_settings.JWT_AUTH_COOKIE:
            try:
                return cookies[api_settings.JWT_AUTH_COOKIE]
            except KeyError:
                raise MissingToken

        raise MissingToken

    def authenticate_credentials(self, payload):
        """
        Returns an active user that matches the payload's user id and email.
        """

        username = self.jwt_get_username_from_payload(payload)

        if not username:
            msg = _('Invalid payload.')
            raise exceptions.AuthenticationFailed(msg)

        try:
            User = get_user_model()
            user = User.objects.get_by_natural_key(username)
        except User.DoesNotExist:
            msg = _('Invalid token.')
            raise exceptions.AuthenticationFailed(msg)

        if not user.is_active:
            msg = _('User account is disabled.')
            raise exceptions.AuthenticationFailed(msg)

        return user

    def authenticate_header(self, request):
        """
        Return a string to be used as the value of the `WWW-Authenticate`
        header in a `401 Unauthenticated` response, or `None` if the
        authentication scheme should return `403 Permission Denied` responses.
        """

        return '{0} realm="{1}"'.format(
            api_settings.JWT_AUTH_HEADER_PREFIX, self.www_authenticate_realm
        )
